import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

public class TestsArgV {
    public static int MAXV = 1_000_000_000;

    public static void main(String[] args) {
        if (args.length > 0) {
            new TestsArgV().gen(args, -1);
        } else {
            new TestsArgV().gen();
        }
    }

    int tests;
    Random rnd;

    public class Equation {
        String variable;
        int value;

        public Equation(String variable, int value) {
            this.variable = variable;
            this.value = value;
        }
    }

    public void writeTest(Equation[][] programs, Equation[] target) {
        try {
            PrintWriter out = tests == -1 ? new PrintWriter(System.out) : new PrintWriter(String.format("%02d", tests));

            HashSet<String> variables = new HashSet<>();
            out.println(programs.length);
            for (Equation[] program : programs) {
                out.println(program.length);
                for (Equation eq : program) {
                    variables.add(eq.variable);
                    out.println(eq.variable + "=" + eq.value);
                }
            }

            ArrayList<Equation> shuffle = new ArrayList<>();
            for (Equation eq : target) {
                if (variables.contains(eq.variable)) {
                    shuffle.add(eq);
                }
            }
            Collections.shuffle(shuffle, rnd);

            out.println(shuffle.size());
            for (Equation eq : shuffle) {
                out.println(eq.variable + " " + eq.value);
            }

            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void sample(int test) {
        rnd = new Random(test);
        if (test == 1) {
            Equation[][] programs = new Equation[][]{
                    {new Equation("a", 1), new Equation("b", 2)},
                    {new Equation("b", 1), new Equation("a", 2)}};
            writeTest(programs, new Equation[]{new Equation("a", 1), new Equation("b", 1)});
        }
        if (test == 2) {
            Equation[][] programs = new Equation[][]{
                    {new Equation("start", 1), new Equation("counter", 1111), new Equation("counter", 10),  new Equation("counter", 3333), new Equation("finish", 1)},
                    {new Equation("start", 2), new Equation("counter", 20), new Equation("counter", 10), new Equation("finish", 2)},
                    {new Equation("start", 3), new Equation("qwerty", 787788), new Equation("finish", 3)}
            };
            writeTest(programs, new Equation[]{new Equation("finish", 1), new Equation("start", 1), new Equation("qwerty", 787788), new Equation("counter", 10)});
        }
        if (test == 3) {
            Equation[][] programs = new Equation[][]{
                    {new Equation("cabababa", 1), new Equation("abacaba", 1), new Equation("babcbab", 2)},
                    {new Equation("babcbab", 1), new Equation("cabababa", 2), new Equation("abacaba", 2)},
                    {new Equation("babcbab", 42), new Equation("abacaba", 42), new Equation("cabababa", 42)}
            };
            writeTest(programs, new Equation[]{new Equation("abacaba", 1), new Equation("babcbab", 2), new Equation("cabababa", 42)});
        }
        Equation[][] programs = new Equation[][]{
                {new Equation("a", 1)}};
        if (test == 22) {
            writeTest(programs, new Equation[]{new Equation("a", 1)});
        }
        if (test == 23) {
            writeTest(programs, new Equation[]{new Equation("a", 1_000_000_000)});
        }
    }

    public String[] getNames(int V, int L) {
        String[] names = new String[V];
        for (int i = 0; i < V; i++) {
            names[i] = "";
            for (int j = 0; j < L; j++) {
                names[i] += (char) ('a' + rnd.nextInt('z' - 'a' + 1));
            }
        }
        return names;
    }

    public void random(int P, int LENP, int LEN, int VARS, int MAXV, boolean bad, int seed) {
        rnd = new Random(seed);

        String[] varNames = getNames(VARS, LEN);

        Equation[] target = new Equation[varNames.length];
        HashMap<String, Equation> targetToPos = new HashMap<>();
        for (int i = 0; i < varNames.length; i++) {
            target[i] = new Equation(varNames[i], -1);
            targetToPos.put(varNames[i], target[i]);
        }
        Equation[][] programs = new Equation[P][LENP];
        for (int i = 0; i < P; i++) {
            for (int j = 0; j < LENP; j++) {
                programs[i][j] = new Equation(varNames[rnd.nextInt(varNames.length)], rnd.nextInt(MAXV) + 1);
            }
        }

        HashSet<String> goodTarget = new HashSet<>();
        HashMap<String, ArrayList<Integer>> allTarget = new HashMap<>();
        int[] eid = new int[programs.length];

        int finished = 0;
        for (int i = 0; i < programs.length; i++) {
            while (eid[i] < programs[i].length && !targetToPos.containsKey(programs[i][eid[i]].variable)) {
                eid[i]++;
            }
            if (eid[i] == programs[i].length) {
                finished++;
                continue;
            }
            Equation variable = programs[i][eid[i]];
            goodTarget.add(variable.variable);
            allTarget.computeIfAbsent(variable.variable, k1 -> new ArrayList<>());
            allTarget.get(variable.variable).add(i);
        }

        while (targetToPos.size() > 0 && goodTarget.size() > 0 && finished < programs.length) {
            HashSet<String> nextGoodTarget = new HashSet<>();
            for (String v : goodTarget) {
                if (!targetToPos.containsKey(v)) {
                    continue;
                }
                ArrayList<Integer> positions = allTarget.get(v);

                if (!bad || rnd.nextInt(100) != 0) {
                    int x = positions.get(rnd.nextInt(positions.size()));
                    targetToPos.get(v).value = programs[x][eid[x]].value;

                    if (rnd.nextBoolean()) {
                        int y = positions.get(rnd.nextInt(positions.size()));
                        programs[y][eid[y]].value = programs[x][eid[x]].value;
                        if (rnd.nextInt(10) == 0) {
                            for (int pos : positions) {
                                programs[pos][eid[pos]].value = programs[x][eid[x]].value;
                            }
                        }
                    }
                } else {
                    targetToPos.get(v).value = rnd.nextInt(MAXV) + 1;
                }

                targetToPos.remove(v);

                for (int p : positions) {
                    if (eid[p] == programs[p].length) {
                        continue;
                    }
                    while (eid[p] < programs[p].length && !targetToPos.containsKey(programs[p][eid[p]].variable)) {
                        eid[p]++;
                    }
                    if (eid[p] == programs[p].length) {
                        finished++;
                        continue;
                    }
                    Equation variable = programs[p][eid[p]];
                    nextGoodTarget.add(variable.variable);
                    allTarget.computeIfAbsent(variable.variable, k1 -> new ArrayList<>());
                    allTarget.get(variable.variable).add(p);
                }
            }
            goodTarget = nextGoodTarget;
        }

        Equation[][] programsRev = new Equation[programs.length][programs[0].length];
        for (int i = 0; i < programsRev.length; i++) {
            for (int j = 0; j < programsRev[i].length; j++) {
                programsRev[i][programsRev[i].length - j - 1] = programs[i][j];
            }
        }

        writeTest(programsRev, target);
    }

    public void oneByOne(int P, int LENP, int LEN, int seed) {
        rnd = new Random(seed);
        String[] varnames = getNames(LENP, LEN);

        Equation[] target = new Equation[LENP];
        Equation[][] programs = new Equation[P][LENP];
        for (int i = 0; i < LENP; i++) {
            for (int j = 0; j < P; j++) {
                programs[j][i] = new Equation(varnames[i], rnd.nextInt(MAXV) + 1);
            }
            target[i] = programs[rnd.nextInt(P)][i];
        }

        writeTest(programs, target);
    }

    public void oneByOneEqual(int P, int LENP, int LEN, int seed) {
        rnd = new Random(seed);
        String[] varnames = getNames(LENP, LEN);

        Equation[] target = new Equation[LENP];
        Equation[][] programs = new Equation[P][LENP];
        for (int i = 0; i < LENP; i++) {
            Equation eq = new Equation(varnames[i], rnd.nextInt(MAXV) + 1);
            for (int j = 0; j < P; j++) {
                programs[j][i] = eq;
            }
            target[i] = eq;
        }

        writeTest(programs, target);
    }

    public void badOneByOne(int P, int LENP, int LEN, int seed) {
        rnd = new Random(seed);
        String[] varnames = getNames(LENP, LEN);

        Equation[] target = new Equation[varnames.length];
        int[] pos = new int[varnames.length];
        Equation[][] programs = new Equation[P][LENP];
        for (int i = 0; i < LENP; i++) {
            for (int j = 0; j < P; j++) {
                programs[j][i] = new Equation(varnames[i], rnd.nextInt(MAXV) + 1);
            }
            pos[i] = rnd.nextInt(P);
            target[i] = programs[pos[i]][i];
        }

        programs[pos[0]][0] = programs[pos[0]][1];
        programs[pos[0]][1] = target[0];
        target[0] = programs[pos[0]][0];
        target[1] = programs[pos[1]][0];

        writeTest(programs, target);
    }

    public void allDifferent(int P, int LENP, int LEN, int seed) {
        rnd = new Random(seed);
        String[] varnames = getNames(LENP * P, LEN);

        Equation[] target = new Equation[varnames.length];
        Equation[][] programs = new Equation[P][LENP];
        for (int i = 0; i < LENP; i++) {
            for (int j = 0; j < P; j++) {
                programs[j][i] = new Equation(varnames[i * P + j], rnd.nextInt(MAXV) + 1);
                target[i * P + j] = programs[j][i];
            }
        }

        writeTest(programs, target);
    }

    public void badAllDifferent(int P, int LENP, int LEN, int seed) {
        rnd = new Random(seed);
        String[] varnames = getNames(LENP * P + 2, LEN);

        Equation[] target = new Equation[varnames.length];
        Equation[][] programs = new Equation[P][LENP];
        for (int i = 0; i < LENP; i++) {
            for (int j = 0; j < P; j++) {
                programs[j][i] = new Equation(varnames[i * P + j], rnd.nextInt(MAXV) + 1);
                target[i * P + j] = programs[j][i];
            }
        }

        int p1 = rnd.nextInt(P);
        int p2 = rnd.nextInt(P);
        programs[p1][0] = new Equation(varnames[varnames.length - 2], rnd.nextInt(MAXV) + 1);
        programs[p1][1] = new Equation(varnames[varnames.length - 1], rnd.nextInt(MAXV) + 1);
        programs[p2][0] = new Equation(varnames[varnames.length - 1], rnd.nextInt(MAXV) + 1);
        programs[p2][1] = new Equation(varnames[varnames.length - 2], rnd.nextInt(MAXV) + 1);

        target[target.length - 2] = programs[p1][0];
        target[target.length - 1] = programs[p2][0];

        writeTest(programs, target);
    }

    public void tl(int P, int LENP, int LEN, int seed) {
        rnd = new Random(seed);
        String[] names = getNames(P * LENP, LEN);

        Equation[] target = new Equation[names.length];
        for (int i = 0; i < names.length; i++) {
            target[i] = new Equation(names[i], rnd.nextInt(MAXV) + 1);
        }
        Equation[][] programs = new Equation[P][LENP];

        int[] eid = new int[P];
        int last = P - 1;
        int curid = 0;
        int i = 0;
        while (true) {
            while (last >= 0 && eid[last] == LENP) {
                last--;
            }
            if (last < 0) {
                break;
            }
            if (curid >= last) {
                curid = 0;
            }

            programs[last][eid[last]] = target[i];
            eid[last]++;

            programs[curid][eid[curid]] = new Equation(target[i].variable, rnd.nextInt(MAXV) + 1);
            eid[curid]++;
            curid++;

            i++;
        }
        for (int j = eid[0]; j < LENP; j++) {
            programs[0][j] = target[i];
            i++;
        }

        Equation[][] programsRev = new Equation[programs.length][programs[0].length];
        for (i = 0; i < programsRev.length; i++) {
            for (int j = 0; j < programsRev[i].length; j++) {
                programsRev[i][programsRev[i].length - j - 1] = programs[i][j];
            }
        }

        writeTest(programsRev, target);
    }

    public void gen(String[] args, int test) {
        tests = test;
        MAXV = Integer.parseInt(args[args.length - 1]);
        switch (args[0]) {
            case "sample":
                sample(Integer.parseInt(args[1]));
                break;
            case "one_by_one":
                oneByOne(Integer.parseInt(args[1]), Integer.parseInt(args[2]), Integer.parseInt(args[3]), Integer.parseInt(args[4]));
                break;
            case "one_by_one_eq":
                oneByOneEqual(Integer.parseInt(args[1]), Integer.parseInt(args[2]), Integer.parseInt(args[3]), Integer.parseInt(args[4]));
                break;
            case "bad_one_by_one":
                badOneByOne(Integer.parseInt(args[1]), Integer.parseInt(args[2]), Integer.parseInt(args[3]), Integer.parseInt(args[4]));
                break;
            case "all_different":
                allDifferent(Integer.parseInt(args[1]), Integer.parseInt(args[2]), Integer.parseInt(args[3]), Integer.parseInt(args[4]));
                break;
            case "bad_all_different":
                badAllDifferent(Integer.parseInt(args[1]), Integer.parseInt(args[2]), Integer.parseInt(args[3]), Integer.parseInt(args[4]));
                break;
            case "random":
                random(Integer.parseInt(args[1]), Integer.parseInt(args[2]), Integer.parseInt(args[3]),
                        Integer.parseInt(args[4]), Integer.parseInt(args[5]), false, Integer.parseInt(args[6]));
                break;
            case "bad_random":
                random(Integer.parseInt(args[1]), Integer.parseInt(args[2]), Integer.parseInt(args[3]),
                        Integer.parseInt(args[4]), Integer.parseInt(args[5]), true, Integer.parseInt(args[6]));
                break;
            case "tl":
                tl(Integer.parseInt(args[1]), Integer.parseInt(args[2]), Integer.parseInt(args[3]), Integer.parseInt(args[4]));
                break;
        }
    }

    public void gen() {
        try (Scanner in = new Scanner(new File("tests.txt"))) {
            int test = 1;
            while (in.hasNext()) {
                String line = in.nextLine();
                if (line.length() == 0)
                    break;
                System.err.println("Generating test: " + line);
                int pos = line.indexOf(" ");
                line = line.substring(pos + 1);
                gen(line.split(" "), test++);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
