#include "testlib.h"
#include <iostream>
#include <cassert>
#include <vector>
#include <complex>

using namespace std;

const long long CMAX = 100'000LL;

typedef complex<long long> point;

static long long cross(const point &a, const point &b) {
    return imag(conj(a) * b);
}

static long long cross(const point &a, const point &b, const point &c) {
    return cross(b - a, c - a);
}

int main(int argc, char* argv[]) {
    registerValidation(argc, argv);
    int n = inf.readInt(3, 10000, "n");
    inf.readEoln();
    vector<point> pnts(n);
    for (int i = 0; i < n; i++) {
        pnts[i].real(inf.readLong(-CMAX, CMAX, "x"));
        inf.readSpace();
        pnts[i].imag(inf.readLong(-CMAX, CMAX, "y"));
        inf.readEoln();
    }
    rotate(pnts.begin(), min_element(pnts.begin(), pnts.end(), [](const point& a, const point& b) {
        return std::make_pair(a.imag(), a.real()) < std::make_pair(b.imag(), b.real());
    }), pnts.end());
    
    vector<point> pt2 = pnts;
    pt2.erase(unique(pt2.begin(), pt2.end()), pt2.end());
    sort(pt2.begin() + 1, pt2.end(), [&](const point& a, const point& b) {
        auto c = cross(a - pnts[0], b - pnts[0]);
        if (c != 0) return c > 0;
        return norm(a - pnts[0]) < norm(b - pnts[0]);
    });
    
    vector<point> hull;
    for (point p : pt2) {
        while (hull.size() >= 2 && cross(hull[hull.size() - 2], hull[hull.size() - 1], p) <= 0) hull.pop_back();
        hull.push_back(p);
    }
    
    ensuref(pnts == hull, "Convex hull differ from initial polygon");
    inf.readEof();
    return 0;
}
