#include "testlib.h"
#include <functional>
#include <cassert>

using namespace std;

int const MAXSUM = 300000;

struct vertex {
	int white;
	int black;
};

struct test {
	vector<vertex> a;
};

bool ALLOW_ZEROS, ALL_PERMUTATIONS;

auto COMP_VERTICES = [](vertex const &a, vertex const &b) {
	if (a.white != b.white) return a.white < b.white;
	return a.black < b.black;
};

vector<vector<int>> genAllPartitions(int n, int sum, int last) {
	if (n == 0) {
		if (sum != 0) return {};
		return {{}};
	}
	vector<vector<int>> res;
	for (int first = 0; first <= last; first++) {
		vector<vector<int>> got = genAllPartitions(n - 1, sum - first, first);
		for (auto &e : got) {
			while ((int) e.size() < first + 1) e.push_back(0);
			e[first]++;
			res.push_back(e);
		}
	}
	return res;
}

vector<test> genByPartitions(int n, vector<int> wP, vector<int> bP) {
	vector<vertex> a(n);
	vector<test> res;	
	function<void(int, int, int)> genPairs = [&](int v, int lastW, int lastB) {
		if (v == n) {
			vector<vertex> b(a.begin(), a.end());
			if (ALL_PERMUTATIONS) {
				sort(b.begin(), b.end(), COMP_VERTICES);
				do {
					res.push_back({b});
				} while (next_permutation(b.begin(), b.end(), COMP_VERTICES));
			} else {
				shuffle(b.begin(), b.end());
				res.push_back({b});
			}
			return;
		}
		int w = 0;
		while (w < (int) wP.size() && wP[w] == 0) ++w;
		assert(w < (int) wP.size());
		for (int b = w == lastW ? lastB : 0; b < (int) bP.size(); b++) {
			if (w == 0 && b == 0 && !ALLOW_ZEROS) continue;
			if (w + b > n - 1) continue;
			if (bP[b] == 0) continue;
			--wP[w];
			--bP[b];
			a[v].white = w;
			a[v].black = b;
			genPairs(v + 1, w, b);
			++wP[w];
			++bP[b];
		}
	};
	genPairs(0, 0, 0);
	return res;
}

vector<test> gen(int n, int whiteSum, int blackSum) {
	vector<vector<int>> whitePartitions = genAllPartitions(n, whiteSum, n - 1);
	vector<vector<int>> blackPartitions = genAllPartitions(n, blackSum, n - 1);
	vector<test> tests;
	set<vector<pair<int, int>>> used;
	for (auto wP : whitePartitions) {
		for (auto bP : blackPartitions) {
			vector<test> g = genByPartitions(n, wP, bP);
			tests.insert(tests.end(), g.begin(), g.end());
		}
	}
	return tests;
}

int main(int argc, char **argv) {
	registerGen(argc, argv, 1);
	int maxN = atoi(argv[1]);
	ALLOW_ZEROS = argc > 2 && atoi(argv[2]) == 1;
	ALL_PERMUTATIONS = argc > 3 && atoi(argv[3]) == 1;
	bool WHITE_LESS_BLACK = argc > 4 && atoi(argv[4]) == 1;
	vector<test> tests;
	int sumN = 0;
	for (int n = maxN; n >= 1; n--) {
		for (int white = 0; white <= n - 1; white++) {
			if (WHITE_LESS_BLACK && white + white > n - 1) {
				break;
			}
			auto g = gen(n, white * 2, 2 * ((n - 1) - white));
			tests.insert(tests.end(), g.begin(), g.end());
			sumN += n * (int) g.size();
		}
	}
	cerr << "Sum of N = " << sumN << endl;
	if (sumN > MAXSUM) {
		while (sumN > MAXSUM) {
			auto &e = tests.back();
			if ((int) e.a.size() == maxN) break;
			sumN -= (int) e.a.size();
			tests.pop_back();
		}
		cerr << "After deleting small: Sum of N = " << sumN << endl;
		if (sumN > MAXSUM) {
			shuffle(tests.begin(), tests.end());
			while (sumN > MAXSUM) {
				auto &e = tests.back();
				sumN -= (int) e.a.size();
				tests.pop_back();
			}
			cerr << "Shuffled, after deleting large: Sum of N = " << sumN << endl;
		}
	}
	reverse(tests.begin(), tests.end());
	cout << tests.size() << '\n';
	for (auto &t : tests) {
		cout << t.a.size() << '\n';
		if (WHITE_LESS_BLACK) {
			if (rnd.next(0, 1) == 0) {
				for (auto &v : t.a) {
					swap(v.white, v.black);
				}
			}
		}
		for (auto &v : t.a) {
			cout << v.white << ' ' << v.black << '\n';
		}
	}
}
