#include "testlib.h"
#include <cassert>
#include <functional>

using namespace std;

struct vertex {
	int white;
	int black;
};

struct test {
	vector<vertex> a;
};

test gen(int n, string op) {
	int sumD = 2 * (n - 1);
	vector<int> deg(n);
	for (int i = 0; i < n; i++) {
		int maxD = sumD - (n - i - 1);
		deg[i] = rnd.next(1, maxD);
		sumD -= deg[i];
	}
	vector<vertex> a(n);
	if (op == "white") {
		for (int i = 0; i < n; i++) {
			a[i].white = deg[i];
			a[i].black = 0;
		}
	} else if (op == "black") {
		for (int i = 0; i < n; i++) {
			a[i].white = 0;
			a[i].black = deg[i];
		}
	} else if (op == "random") {
		for (int i = 0; i < n; i++) {
			a[i].white = rnd.next(0, deg[i]);
			a[i].black = deg[i] - a[i].white;
		}
	} else assert(false);
	shuffle(a.begin(), a.end());
	return {a};
}

vector<string> TYPES = {"white", "black", "random" };

int main(int argc, char **argv) {
	registerGen(argc, argv, 1);
	int sumN = opt<int>("sumN");
	int maxN = has_opt("n") ? opt<int>("n") : 0;
	bool make_max = opt<bool>("max");
	string type = opt<string>("type");
	vector<test> tests;
	int currentSum = 0;
	while (currentSum < sumN) {
		int n = make_max ? sumN : max(1, min(rnd.next(maxN / 2, maxN), sumN - currentSum));
		string op = type == "any" ? TYPES[rnd.next(0, (int) TYPES.size() - 1)] : type;
		tests.push_back(gen(n, op));
		currentSum += n;
	}
	cout << tests.size() << '\n';
	for (auto &e : tests) {
		cout << e.a.size() << '\n';
		for (auto &v : e.a) {
			cout << v.white << ' ' << v.black << '\n';
		}
	}
}
